from requestsgarant import RequestsGarant
from bs4 import BeautifulSoup
import os
import csv


class ParserHidemy(RequestsGarant):

    def __init__(self, params):
        super().__init__()
        self.headers = {'content-type': 'text/plain;charset=UTF-8',
                        'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/102.0.5005.63 Safari/537.36',
                        }
        self.params = params
        self.url = 'https://hidemy.name/ru/proxy-list/'
        self.method = 'get'
        self.finish = False

    def do_status_code(self):
        if (soup := BeautifulSoup(self.response.content, 'lxml')) is not None:
            if (table_soup := soup.find("div", class_="table_block")) is not None:
                tr_soup = table_soup.find_all('tr')
                if len(tr_soup) == 1:
                    self.finish = True
                    return f"Последняя таблица"
                elif len(tr_soup) > 1:
                    with open(f'{os.path.abspath(os.curdir)}/{self.__class__.__name__}.csv', 'a',
                              encoding='utf-8') as file:
                        writer = csv.writer(file)
                        for td_soup in tr_soup:
                            self.write_row(writer, td_soup)
                    return f"записываем {len(tr_soup)}"
                else:
                    return f"tr_soup меньше 1: {len(tr_soup)}"

            return f"table_block отсутствует {self.response.text}"
        return f"нет супа {self.response.text}"

    @staticmethod
    def write_row(writer, tr_soup):
        try:
            writer.writerow([tr_soup.text for tr_soup in tr_soup.find_all('td')])
        except Exception as e:
            print(e)


def start_parser_hidemi(params, cls=ParserHidemy):
    for start in range(0, 17000, 64):
        params.update({'start': start})
        obj = cls(params).get_obj_rezult()
        print(f"start={start}, {obj.rezult}")
        if obj.finish:
            return start


class HidemyApi(RequestsGarant):
    def __init__(self, params):
        super().__init__()
        self.params = params
        self.url = f'http://hidemy.name/ru/api/proxylist.php'
        self.method = 'get'





from unittest import TestCase
from requests.models import Response
import os
from src.parserproxy import *
import json

def get_requestsgarant_ob(page_html, requestsgarant_ob):
    requestsgarant_ob.response = Response()
    with open(f'{os.path.abspath(os.curdir)}\\pages\\{page_html}', 'rb') as file:
        requestsgarant_ob.response._content = file.read()
    return requestsgarant_ob

#
# def get_response_content_hidemy_from_file():
#     ph = ParserHidemy({'type': 'h'})
#     pass

class TestTestCase(TestCase):

    def test_get_requestsgarant_ob(self):
        '''
        тест на get_requestsgarant_ob
        :return:
        '''
        requestsgarant_ob = get_requestsgarant_ob('hidemy.name_64_proxy.html', ParserHidemy({}))
        print(requestsgarant_ob.response.content)

    def test_0(self):
        for start in range(0, 15000, 64):
            print({"start": start})

    def test(self):
        start_parser_hidemi({'type': 'h'})

    def test_1(self):
        with open(f'{os.path.abspath(os.curdir)}\\pages\\{"hidemy.name_64_proxy.html"}', 'rb') as file:

            if (soup := BeautifulSoup(file.read(), 'lxml')) is not None:
                if (table_soup := soup.find("div", class_="table_block")) is not None:
                    tr_soup = table_soup.find_all('tr')
                    if len(tr_soup) == 1:
                        return f"Последняя таблица"
                    elif len(tr_soup) > 1:

                            for td_soup in tr_soup:
                                print(td_soup)


class ParserHidemyTestCase(TestCase):
    @staticmethod
    def write_row(writer, tr_soup):
        print([tr_soup.text for tr_soup in tr_soup.find_all('td')])


    def get_ob_parser_hidemy(self, page_html):
        requestsgarant_ob = get_requestsgarant_ob(page_html, ParserHidemy({}))
        requestsgarant_ob.write_row = self.write_row
        return requestsgarant_ob

    def test_hidemy_name_64_proxy(self):
        ob = self.get_ob_parser_hidemy('hidemy.name_64_proxy.html')

        print(ob.do_status_code())
        print(ob.finish)


    def test_hidemy_name_0_proxy(self):
        ob = self.get_ob_parser_hidemy('hidemy.name_0_proxy.html')
        print(ob.do_status_code())
        print(ob.finish)


    def test_ParserHidemy(self):
        ob = ParserHidemy({'type': 'h'})
        ob.write_row = self.write_row
        print(ob.get_rezult())


class HidemyApiTestCase(TestCase):
    def setUp(self) -> None:
        from dotenv import load_dotenv, dotenv_values

        load_dotenv()

        if 'PATH_TO_ENV' in dotenv_values(".env"):
            load_dotenv(dotenv_path=os.environ.get('PATH_TO_ENV'))
        else:
            load_dotenv(dotenv_path='C:\\config_bank_rko\\.env')

        self.code = os.environ.get('hidemy')

    def test_0(self):

        obj = HidemyApi({'out': 'js', 'code': self.code, 'type': 'h'})
        obj.get_obj_rezult()
        print(obj.response)
        with open('HidemyApi.json', 'w', encoding='utf-8') as file:
            file.write(json.dumps(obj.response_json))


    def test_1(self):
        with open('HidemyApi.json', 'r', encoding='utf-8') as file:
            print(json.loads(file.read())[0])












import time

import requests
import traceback


def return_test_response(json_data: dict, status_code: int = 200):
    import json
    from requests.models import Response
    the_response = Response()
    the_response.code = "expired"
    the_response.error_type = "expired"
    the_response.status_code = status_code
    the_response._content = json.dumps(json_data).encode('utf-8')
    return the_response


class RequestsGarant:
    """Кастомная библиотека для запросов"""

    def __init__(self):
        self.success = False
        self.rezult = None
        self.custom_test = False
        self.json = None
        self.data = None
        self.timeout = 50
        self.resend_send = False

        self.connection_error = False
        self.timeot_error = False

    def get_guarantee_response(self):
        try:
            return self.get_response_functions()()
        except requests.exceptions.Timeout:
            self.timeot_error = True
            self.resend_send = True
            return f"Превышен таймаут {self.timeout}"
        except requests.exceptions.ConnectionError:
            self.resend_send = True
            self.connection_error = True
            return "Ошибка подключения"
        except:
            return traceback.format_exc()

    def do_json(self):
        pass

    def do_status_code(self):
        if self.response_status_code == 429:
            self.resend_send = True
            time.sleep(10)
        return self.response_status_code

    def rezult_function(self):
        pass

    def set_null_attr(self, *args_init):
        # обнуление атрибутов
        for attr in [attr for attr in self.__dict__.keys()]:
            args = (self, attr)
            if hasattr(*args):
                delattr(*args)
        self.__init__(*args_init)

    def get_rezult(self):
        self.response = self.get_guarantee_response()
        if type(self.response) is type(requests.models.Response()):
            try:
                self.response_json = self.response.json()
                do_json = self.do_json()
                if do_json is not None:
                    self.rezult = do_json
                    return do_json
                else:
                    self.rezult = self.response_json
                    return self.response_json
            except:
                self.response_status_code = self.response.status_code
                do_status_code = self.do_status_code()
                if do_status_code is None:
                    self.rezult = self.response_status_code
                else:
                    self.rezult = do_status_code
                return do_status_code
        else:
            self.rezult = self.response
            return self.response

    def get_obj_rezult(self):
        self.get_rezult()
        return self

    def return_test_response(self):
        self.define_json_response_test()
        return return_test_response(self.json_response_test)

    def define_json_response_test(self):
        self.json_response_test = {"mess": 'заглушка'}

    def get_response_functions(self):
        self.args_request = {attr: getattr(self, attr) for attr in
                             ['method', 'params', 'url', 'proxies', 'data', 'json', 'headers', 'timeout', 'files', 'verify', 'cert']
                             if hasattr(self, attr) is True and getattr(self, attr) is not None
                             if getattr(self, 'method') == 'get' and attr != 'json'
                             or
                             getattr(self, 'method') == 'post' and attr != 'params'}

        if (self.custom_test and self.test) is True:
            return self.return_test_response
        else:
            return self.get_response_production

    def get_response_production(self):
        return requests.request(**self.args_request)


class RequestsGarantSession(RequestsGarant):
    def __init__(self, session):
        super().__init__()
        self.session = session

    def get_response_production(self):
        return self.session.request(**self.args_request)

class RequestsGarantTest(RequestsGarant):
    def __init__(self):
        super().__init__()

    def get_response_functions(self):
        if self.test is True:
            self.assign_test()
        else:
            self.assign_production()

        return super().get_response_functions()


class RequestsGarantTestBaseUrl(RequestsGarantTest):
    def __init__(self):
        super().__init__()

    def assign_test(self):
        self.url = f"{self.base_url_test}{self.endpoint}"

    def assign_production(self):
        self.url = f"{self.base_url}{self.endpoint}"


def assign_url_endpoint(obj_cuctom_request):
    if obj_cuctom_request.test is True:
        obj_cuctom_request.url = f"{obj_cuctom_request.url}{obj_cuctom_request.test_endpoint}"
    else:
        obj_cuctom_request.url = f"{obj_cuctom_request.url}{obj_cuctom_request.endpoint}"


class RequestsGarantTestEndpoint(RequestsGarantTest):
    def __init__(self):
        super().__init__()

    def assign_test(self):
        self.url = f"{self.base_url}{self.endpoint_test}"

    def assign_production(self):
        self.url = f"{self.base_url}{self.endpoint}"


class RequestsGarantTestHeaders(RequestsGarantTest):
    def __init__(self):
        super().__init__()

    def assign_test(self):
        self.headers.update(self.dict_key_test)

    def assign_production(self):
        self.headers.update(self.dict_key)


class RequestsGarantTestJson(RequestsGarantTest):
    pass
















